/*
  original
  https://ccrma.stanford.edu/~fgeorg/250a/lab2/arduino-0019/libraries/LiquidCrystal/LiquidCrystal.h
  https://ccrma.stanford.edu/~fgeorg/250a/lab2/arduino-0019/libraries/LiquidCrystal/LiquidCrystal.cpp
*/

#include "esp_common.h"

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"
#include "freertos/portmacro.h"

#include "micro_delay.h"
//#include "gpio.h"
#include "i2c.h"

#include "LcdAcm1602ni.h"

// commands
#define LCD_CLEARDISPLAY 0x01
#define LCD_RETURNHOME 0x02
#define LCD_ENTRYMODESET 0x04
#define LCD_DISPLAYCONTROL 0x08
#define LCD_CURSORSHIFT 0x10
#define LCD_FUNCTIONSET 0x20
#define LCD_SETCGRAMADDR 0x40
#define LCD_SETDDRAMADDR 0x80

#define LCD_EX_SETBIASOSC       0x10        // Bias selection / Internal OSC frequency adjust
#define LCD_EX_SETICONRAMADDR   0x40        // Set ICON RAM address
#define LCD_EX_POWICONCONTRASTH 0x50        // Power / ICON control / Contrast set(high byte)
#define LCD_EX_FOLLOWERCONTROL  0x60        // Follower control
#define LCD_EX_CONTRASTSETL     0x70        // Contrast set(low byte)

// flags for display entry mode
#define LCD_ENTRYRIGHT 0x00
#define LCD_ENTRYLEFT 0x02
#define LCD_ENTRYSHIFTINCREMENT 0x01
#define LCD_ENTRYSHIFTDECREMENT 0x00

// flags for display on/off control
#define LCD_DISPLAYON 0x04
#define LCD_DISPLAYOFF 0x00
#define LCD_CURSORON 0x02
#define LCD_CURSOROFF 0x00
#define LCD_BLINKON 0x01
#define LCD_BLINKOFF 0x00

// flags for display/cursor shift
#define LCD_DISPLAYMOVE 0x08
#define LCD_CURSORMOVE 0x00
#define LCD_MOVERIGHT 0x04
#define LCD_MOVELEFT 0x00

// flags for function set
#define LCD_8BITMODE 0x10
#define LCD_4BITMODE 0x00
#define LCD_2LINE 0x08
#define LCD_1LINE 0x00
#define LCD_5x10DOTS 0x04
#define LCD_5x8DOTS 0x00
#define LCD_EX_INSTRUCTION      0x01        // IS: instruction table select

// flags for Bias selection
#define LCD_BIAS_1_4            0x08        // bias will be 1/4
#define LCD_BIAS_1_5            0x00        // bias will be 1/5

// flags Power / ICON control / Contrast set(high byte)
#define LCD_ICON_ON             0x08        // ICON display on
#define LCD_ICON_OFF            0x00        // ICON display off
#define LCD_BOOST_ON            0x04        // booster circuit is turn on
#define LCD_BOOST_OFF           0x00        // booster circuit is turn off
#define LCD_OSC_122HZ           0x00        // 122Hz@3.0V
#define LCD_OSC_131HZ           0x01        // 131Hz@3.0V
#define LCD_OSC_144HZ           0x02        // 144Hz@3.0V
#define LCD_OSC_161HZ           0x03        // 161Hz@3.0V
#define LCD_OSC_183HZ           0x04        // 183Hz@3.0V
#define LCD_OSC_221HZ           0x05        // 221Hz@3.0V
#define LCD_OSC_274HZ           0x06        // 274Hz@3.0V
#define LCD_OSC_347HZ           0x07        // 347Hz@3.0V

// flags Follower control
#define LCD_FOLLOWER_ON         0x08        // internal follower circuit is turn on
#define LCD_FOLLOWER_OFF        0x00        // internal follower circuit is turn off
#define LCD_RAB_1_00            0x00        // 1+(Rb/Ra)=1.00
#define LCD_RAB_1_25            0x01        // 1+(Rb/Ra)=1.25
#define LCD_RAB_1_50            0x02        // 1+(Rb/Ra)=1.50
#define LCD_RAB_1_80            0x03        // 1+(Rb/Ra)=1.80
#define LCD_RAB_2_00            0x04        // 1+(Rb/Ra)=2.00
#define LCD_RAB_2_50            0x05        // 1+(Rb/Ra)=2.50
#define LCD_RAB_3_00            0x06        // 1+(Rb/Ra)=3.00
#define LCD_RAB_3_75            0x07        // 1+(Rb/Ra)=3.75


#define LOW 0
#define HIGH 1

static void LCD_ACM1602NI_command(uint8_t value) ;
static void LCD_ACM1602NI_data(uint8_t value) ;
static void LCD_ACM1602NI_wierWrite(uint8_t value) ;
static void LCD_ACM1602NI_lcd_out (uint8_t dat, uint8_t rs) ;
static void LCD_DelayMicroseconds( uint32_t a );
static void extendFunctionSet() ;
static void normalFunctionSet() ;
static void setEntryMode(uint8_t setBit) ;



// When the display powers up, it is configured as follows:
//
// 1. Display clear
// 2. Function set: 
//    DL = 1; 8-bit interface data 
//    N = 0; 1-line display 
//    F = 0; 5x8 dot character font 
// 3. Display on/off control: 
//    D = 0; Display off 
//    C = 0; Cursor off 
//    B = 0; Blinking off 
// 4. Entry mode set: 
//    I/D = 1; Increment by 1 
//    S = 0; No shift 
//
// Note, however, that resetting the Arduino doesn't reset the LCD, so we
// can't assume that its in that state when a sketch starts (and the
// LiquidCrystal constructor is called).



#define TYPE_ST7032 //AE_AQM0802

#ifdef TYPE_ST7032
#define ST7032_I2C_DEFAULT_ADDR     0x7C  // 7c 01111100 3e 0011 1110
#define I2C_SLAVE_ADRS ( ST7032_I2C_DEFAULT_ADDR )
#define LCD_ACM1602NI_RSBIT 0x40
#else
#define ACM1602N1_FLW_FBW_ADDR     0xA0
#define I2C_SLAVE_ADRS ( ACM1602N1_FLW_FBW_ADDR )
#define LCD_ACM1602NI_RSBIT 0x80
#endif



char _displaycontrol ;
char _displaymode ;

void LCD_ACM1602NI_String( char* a )
{
    for( ; *a != 0 ; a ++ ) {
        LCD_ACM1602NI_outchar( *a ) ;
        LCD_DelayMicroseconds(60);
    }
}

char _displayfunction ;

/********** high level commands, for the user! */
void LCD_ACM1602NI_clear()
{
    LCD_ACM1602NI_command(LCD_CLEARDISPLAY);  // clear display, set cursor position to zero
    LCD_DelayMicroseconds(2000);  // this command takes a long time!
}

void LCD_ACM1602NI_home()
{
    LCD_ACM1602NI_command(LCD_RETURNHOME);  // set cursor position to zero
    LCD_DelayMicroseconds(2000);  // this command takes a long time!
}

void LCD_ACM1602NI_setCursor(uint8_t col, uint8_t row)
{
    if( row == 0 ) {
        LCD_ACM1602NI_command(LCD_SETDDRAMADDR | (col) );
    }
    else {
        LCD_ACM1602NI_command(LCD_SETDDRAMADDR | (col + 0x40) );
    }
    LCD_DelayMicroseconds(100);
}

void LCD_ACM1602NI_noDisplay() {
    _displaycontrol &= ~LCD_DISPLAYON;
    LCD_ACM1602NI_command(LCD_DISPLAYCONTROL | _displaycontrol);
    LCD_DelayMicroseconds(60);
}
void LCD_ACM1602NI_display() {
    _displaycontrol |= LCD_DISPLAYON;
    LCD_ACM1602NI_command(LCD_DISPLAYCONTROL | _displaycontrol);
    LCD_DelayMicroseconds(60);
}

void LCD_ACM1602NI_noCursor() {
    _displaycontrol &= ~LCD_CURSORON;
    LCD_ACM1602NI_command(LCD_DISPLAYCONTROL | _displaycontrol);
    LCD_DelayMicroseconds(60);
}
void LCD_ACM1602NI_cursor() {
    _displaycontrol |= LCD_CURSORON;
    LCD_ACM1602NI_command(LCD_DISPLAYCONTROL | _displaycontrol);
    LCD_DelayMicroseconds(60);
}

void LCD_ACM1602NI_noBlink() {
    _displaycontrol &= ~LCD_BLINKON;
    LCD_ACM1602NI_command(LCD_DISPLAYCONTROL | _displaycontrol);
    LCD_DelayMicroseconds(60);
}
void LCD_ACM1602NI_blink() {
    _displaycontrol |= LCD_BLINKON;
    LCD_ACM1602NI_command(LCD_DISPLAYCONTROL | _displaycontrol);
    LCD_DelayMicroseconds(60);
}

void LCD_ACM1602NI_createChar(uint8_t location, uint8_t charmap[]) {
    char i ;
    location &= 0x7; // we only have 8 locations 0-7
    LCD_ACM1602NI_command(LCD_SETCGRAMADDR | (location << 3));
    LCD_DelayMicroseconds(60);
    for ( i=0; i<8; i++) {
        LCD_ACM1602NI_data(charmap[i]);
        LCD_DelayMicroseconds(60);
    }
}
void LCD_ACM1602NI_setContrast(uint8_t cont)
{
    extendFunctionSet();
    LCD_ACM1602NI_command(LCD_EX_CONTRASTSETL | (cont & 0x0f));                     // Contrast set
    LCD_DelayMicroseconds(200);                          // Wait time >200ms (for power stable)
    LCD_ACM1602NI_command(LCD_EX_POWICONCONTRASTH | LCD_ICON_ON | LCD_BOOST_ON | ((cont >> 4) & 0x03)); // Power, ICON, Contrast control
    LCD_DelayMicroseconds(200);                          // Wait time >200ms (for power stable)
    normalFunctionSet();
}
void LCD_ACM1602NI_init( void )
{
    _displayfunction = LCD_8BITMODE | LCD_2LINE | LCD_5x8DOTS;   
    //_displayfunction |= LCD_5x10DOTS;

    _displaycontrol = LCD_DISPLAYON | LCD_CURSOROFF | LCD_BLINKOFF;  

    LCD_DelayMicroseconds(50000); 

#ifdef TYPE_ST7032
    normalFunctionSet();
#endif

#ifdef TYPE_ST7032
    extendFunctionSet();
    LCD_DelayMicroseconds(200);                          // Wait time >200ms (for power stable)
    LCD_ACM1602NI_command(LCD_EX_SETBIASOSC | LCD_BIAS_1_5 | LCD_OSC_183HZ);            // 1/5bias, OSC=183Hz@3.0V
    LCD_DelayMicroseconds(200);                          // Wait time >200ms (for power stable)
    LCD_ACM1602NI_command(LCD_EX_FOLLOWERCONTROL | LCD_FOLLOWER_ON | LCD_RAB_2_00);     // internal follower circuit is turn on
    LCD_DelayMicroseconds(200);                          // Wait time >200ms (for power stable)
    normalFunctionSet();
    LCD_DelayMicroseconds(200);                          // Wait time >200ms (for power stable)

    LCD_ACM1602NI_setContrast(30);

    normalFunctionSet();

    //extendFunctionSet(); //http://kingyonull.blogspot.jp/2013/06/i2caqm0802a-rn-gbw.html
    //LCD_ACM1602NI_command(0x14);//internal osc
    //LCD_ACM1602NI_command(0x70);//contrast
    //LCD_ACM1602NI_command(0x56);//power/icon
    //LCD_ACM1602NI_command(0x6c);//follower
    //normalFunctionSet();

#endif

    LCD_ACM1602NI_command(LCD_FUNCTIONSET | _displayfunction);
    LCD_DelayMicroseconds(4500);  // wait more than 4.1ms

    LCD_ACM1602NI_command(LCD_FUNCTIONSET | _displayfunction);//もう一度
    LCD_DelayMicroseconds(150);

    LCD_ACM1602NI_command(LCD_FUNCTIONSET | _displayfunction);
    LCD_DelayMicroseconds(150);

    //    LCD_ACM1602NI_command(LCD_FUNCTIONSET | _displayfunction);
    //    LCD_DelayMicroseconds(150);

    //LCD_ACM1602NI_command(LCD_ENTRYMODESET | LCD_ENTRYLEFT | LCD_ENTRYSHIFTDECREMENT);
    //LCD_DelayMicroseconds(150);

    _displaymode      = 0x00;//LCD_ENTRYLEFT | LCD_ENTRYSHIFTDECREMENT;

    LCD_ACM1602NI_command(LCD_ENTRYMODESET | LCD_ENTRYLEFT | LCD_ENTRYSHIFTDECREMENT);
    LCD_DelayMicroseconds(150);

    LCD_ACM1602NI_clear();
    //setEntryMode(LCD_ENTRYLEFT | LCD_ENTRYSHIFTDECREMENT);
}



static void setEntryMode(uint8_t setBit) {
  _displaymode |= setBit;
  LCD_ACM1602NI_command(LCD_ENTRYMODESET | _displaymode);
  LCD_DelayMicroseconds(200);                          // Wait time >200ms (for power stable)
}




static void extendFunctionSet() {
  LCD_ACM1602NI_command(LCD_FUNCTIONSET | _displayfunction | LCD_EX_INSTRUCTION);
  LCD_DelayMicroseconds(200);                          // Wait time >200ms (for power stable)
}
static void normalFunctionSet() {
  LCD_ACM1602NI_command(LCD_FUNCTIONSET | _displayfunction);
  LCD_DelayMicroseconds(200);                          // Wait time >200ms (for power stable)
}


/*********** mid level commands, for sending data/cmds */
static void LCD_ACM1602NI_command(uint8_t value) {
    LCD_ACM1602NI_lcd_out(value, LOW);
}
static void LCD_ACM1602NI_data(uint8_t value) {
    LCD_ACM1602NI_lcd_out(value, HIGH);
}
void LCD_ACM1602NI_outchar(uint8_t value) {
    LCD_ACM1602NI_lcd_out(value, HIGH);
}

// Send 1byte Data to I2C LCD 
static void LCD_ACM1602NI_lcd_out (uint8_t dat, uint8_t rs) {
    if( i2c_start() == false ) {
        printf( "i2c_start error\n" ) ;
    }
    LCD_DelayMicroseconds( 10 ) ;
    LCD_ACM1602NI_wierWrite( I2C_SLAVE_ADRS ) ;
    LCD_DelayMicroseconds( 10 ) ;
    if(rs == LOW){
        LCD_ACM1602NI_wierWrite(0x00);
    } else {
        LCD_ACM1602NI_wierWrite(LCD_ACM1602NI_RSBIT);
    }
    LCD_DelayMicroseconds( 10 ) ;
    LCD_ACM1602NI_wierWrite(dat);
    LCD_DelayMicroseconds( 10 ) ;
    if( i2c_stop() == false ) {
        i2c_stop();
        printf( "i2c_stop() error\n");
    }
    //Wire.endTransmission();
}
static void LCD_ACM1602NI_wierWrite(uint8_t value) {
    bool ret = i2c_write( value ) ;
}
static void LCD_DelayMicroseconds( uint32_t a )
{
    micro_delay( a ) ;
}
