/******************************************************************************
 * Copyright 2013-2014 Espressif Systems (Wuxi)
 *
 * FileName: user_main.c
 *
 * Description: entry file of user application
 *
 * Modification history:
 *     2014/12/1, v1.0 create this file.
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "esp_common.h"
#include "ets_sys.h"

//FreeRTOS SDK の ets_sys.h が、古いバージョンなので、esp_iot_sdk_v1.4.0 の ets_sys.h から必要な物を持ってきた(以下2行)
//_xt_isr_mask, _xt_isr_unmask は、将来 os_isr_mask os_isr_unmask になる可能性がある。
#define ETS_GPIO_INTR_DISABLE(); _xt_isr_mask(1<<ETS_GPIO_INUM);
#define ETS_GPIO_INTR_ENABLE(); _xt_isr_unmask(1<<ETS_GPIO_INUM);

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"
#include "freertos/portmacro.h"

#include "lwip/sockets.h"
#include "lwip/dns.h"
#include "lwip/netdb.h"
#include "micro_delay.h"
#include "gpio.h"
#include "i2c.h"
#include "LcdAcm1602ni.h"

static void ICACHE_FLASH_ATTR initGpioInterrupt( void ) ;

static void ICACHE_FLASH_ATTR init_gpio15( void ) ;
static void ICACHE_FLASH_ATTR pgio15( char onOff );
static /* IRAM_FUNC_ATTR */  doppler_int_handler(void* org);

volatile short g_counter ;

#define server_ip "192.168.101.142"
#define server_port 9669

void ICACHE_FLASH_ATTR smartconfig_done(sc_status status, void *pdata)
{
    switch(status) {
        case SC_STATUS_WAIT:
            printf("SC_STATUS_WAIT\n");
            break;
        case SC_STATUS_FIND_CHANNEL:
            printf("SC_STATUS_FIND_CHANNEL\n");
            break;
        case SC_STATUS_GETTING_SSID_PSWD:
            printf("SC_STATUS_GETTING_SSID_PSWD\n");
            sc_type *type = pdata;
            if (*type == SC_TYPE_ESPTOUCH) {
                printf("SC_TYPE:SC_TYPE_ESPTOUCH\n");
            } else {
                printf("SC_TYPE:SC_TYPE_AIRKISS\n");
            }
            break;
        case SC_STATUS_LINK:
            printf("SC_STATUS_LINK\n");
            struct station_config *sta_conf = pdata;
	
	        wifi_station_set_config(sta_conf);
	        wifi_station_disconnect();
	        wifi_station_connect();
            break;
        case SC_STATUS_LINK_OVER:
            printf("SC_STATUS_LINK_OVER\n");
            if (pdata != NULL) {
                uint8 phone_ip[4] = {0};

                memcpy(phone_ip, (uint8*)pdata, 4);
                printf("Phone ip: %d.%d.%d.%d\n",phone_ip[0],phone_ip[1],phone_ip[2],phone_ip[3]);
            }
            smartconfig_stop();
            break;
    }
	
}

void ICACHE_FLASH_ATTR smartconfig_task(void *pvParameters)
{
    smartconfig_start(smartconfig_done);
    vTaskDelete(NULL);
}

static void ICACHE_FLASH_ATTR task2(void *pvParameters)
{
    volatile short loopCnt = 0 ;
    char numBuf[ 8 ] ;
    printf("Hello, welcome to task2!\r\n");
    //vTaskDelay( 20000 ) ;
    while (1) {
        printf( "task2(%d) intrpt %d\n", loopCnt, g_counter) ;

        LCD_ACM1602NI_setCursor(0,0);
        sprintf( numBuf, "L:%d", loopCnt ++ );
        LCD_ACM1602NI_String( numBuf ) ;

        LCD_ACM1602NI_setCursor(0,1);
        sprintf( numBuf, "I:%d", g_counter);
        LCD_ACM1602NI_String( numBuf ) ;

        vTaskDelay( 50 ) ;
    }
}

static void ICACHE_FLASH_ATTR task3(void *pvParameters)
{
    uint32_t last ;
    printf("Hello, welcome to task3!\r\n");
    last = system_get_time() ;
    while (1) {//scl sda OK
        vTaskDelay(100) ;
        pgio15(1);
        printf("hi\n");

        vTaskDelay(100) ;
        pgio15(0);
        printf("low\n");
    }
}

/******************************************************************************
 * FunctionName : user_init
 * Description  : entry of user application, init user function here
 * Parameters   : none
 * Returns      : none
*******************************************************************************/
void ICACHE_FLASH_ATTR user_init(void)
{
    uart_init_new();

    //uart 関連のポートの設定は、uart_init_new() の後で行う
    //ESP8266のAPIのuart_init関数は、なぜか、UART0とUART1を同時に初期化するという仕組みになっています。そのためUART0だけ初期化したい場合でも、UART1まで初期化してしまいます。
    //Set GPIO2(UART1_TXD) to output mode (I2C SDA で使用している)
    PIN_FUNC_SELECT(PERIPHS_IO_MUX_GPIO2_U, FUNC_GPIO2);
    //Set GPIO2(UART1_TXD) low (I2C SDA で使用している)
    gpio_output_set(0, BIT2, BIT2, 0);

    i2c_init();

    LCD_ACM1602NI_init() ;
    LCD_ACM1602NI_display() ;

    LCD_ACM1602NI_setCursor(0,0);
    LCD_ACM1602NI_String( "1234567" ) ;
    printf("SDK version:%s\n", system_get_sdk_version());
    init_gpio15();

    initGpioInterrupt();

    xTaskCreate(task2, "task2", 256, NULL, 2, NULL);
    xTaskCreate(task3, "task3", 256, NULL, 2, NULL);

    /* need to set opmode before you set config */
    wifi_set_opmode(STATION_MODE);


    xTaskCreate(smartconfig_task, "smartconfig_task", 256, NULL, 2, NULL);
}

static void ICACHE_FLASH_ATTR init_gpio15( void )
{
    GPIO_ConfigTypeDef  pGPIOConfig ;
    pGPIOConfig.GPIO_Pin = GPIO_Pin_15 ; //GPIO_Pin_0〜GPIO_Pin_15. or でつなげると、同じ条件のポートを一括で設定できる。
    pGPIOConfig.GPIO_Mode = GPIO_Mode_Output ; //GPIO_Mode_Input, GPIO_Mode_Out_OD, GPIO_Mode_Output, GPIO_Mode_Sigma_Delta
    pGPIOConfig.GPIO_Pullup = GPIO_PullUp_DIS ; //GPIO_PullUp_DIS, GPIO_PullUp_EN
    pGPIOConfig.GPIO_IntrType = GPIO_PIN_INTR_DISABLE ; //GPIO_PIN_INTR_DISABLE, GPIO_PIN_INTR_POSEDGE, GPIO_PIN_INTR_NEGEDGE, GPIO_PIN_INTR_ANYEGDE, GPIO_PIN_INTR_LOLEVEL, GPIO_PIN_INTR_HILEVEL
    gpio_config( &pGPIOConfig ) ;
    pgio15(0);
}
static void ICACHE_FLASH_ATTR pgio15( char onOff )
{
    if( onOff ) {
        GPIO_REG_WRITE(GPIO_OUT_W1TS_ADDRESS, GPIO_Pin_15) ;
    }
    else {
        GPIO_REG_WRITE(GPIO_OUT_W1TC_ADDRESS, GPIO_Pin_15) ;
    }
}

//以下、割り込み
static /* IRAM_FUNC_ATTR */  doppler_int_handler(void* org){
    pgio15(1);
    uint32_t gpio_status = GPIO_REG_READ(GPIO_STATUS_ADDRESS);//gpio_statusに、GPIO割り込み要因のビットが立っている。
    g_counter ++ ;
    // 全てのピン(GPIO0〜GPIO15)の割り込みは、ここに来るので、複数のGPIO割り込みを使う場合には、
    // GPIO_Pin_0〜GPIO_Pin_15 bit の状態で処理を分ける必要がある。
    //GPIO_REG_WRITE(GPIO_STATUS_W1TC_ADDRESS, gpio_status);//割り込み要因クリア
    GPIO_REG_WRITE(GPIO_STATUS_W1TC_ADDRESS, 0xffff);//割り込み要因クリア
}

static void ICACHE_FLASH_ATTR initGpioInterrupt( void ) {
    GPIO_ConfigTypeDef  pGPIOConfig ;
    pGPIOConfig.GPIO_Pin = GPIO_Pin_5 ; //GPIO_Pin_0〜GPIO_Pin_15. or でつなげると、同じ条件のポートを一括で設定できる。
    pGPIOConfig.GPIO_Mode = GPIO_Mode_Input ; //GPIO_Mode_Input, GPIO_Mode_Out_OD, GPIO_Mode_Output, GPIO_Mode_Sigma_Delta
    pGPIOConfig.GPIO_Pullup = GPIO_PullUp_EN ; //GPIO_PullUp_DIS, GPIO_PullUp_EN
    //pGPIOConfig.GPIO_IntrType = GPIO_PIN_INTR_NEGEDGE ; //GPIO_PIN_INTR_DISABLE, GPIO_PIN_INTR_POSEDGE, GPIO_PIN_INTR_NEGEDGE, GPIO_PIN_INTR_ANYEGDE, GPIO_PIN_INTR_LOLEVEL, GPIO_PIN_INTR_HILEVEL
    pGPIOConfig.GPIO_IntrType = GPIO_PIN_INTR_NEGEDGE ;

    //ETS_GPIO_INTR_DISABLE(); ★★これがあるとハングアップする★★ なんで????!!!

    gpio_config( &pGPIOConfig ) ;

    GPIO_REG_WRITE(GPIO_STATUS_W1TC_ADDRESS, 0xffff);//GPIO の割り込み要因クリア
    gpio_intr_handler_register(doppler_int_handler);//GPIO の割り込みベクターは1つだけ。複数のGPIO割り込みで別動作をさせたい場合には、割り込み要因をチェックすること

    ETS_GPIO_INTR_ENABLE() ;

}
